;**************************************************************************************
;PROGRAM		:	VF CONTROL OF 3-PHASE INDUCTION MOTOR (ADC SUBROUTINE)
;MICROCONTROLLER	:	PIC16F777
;CRYSTAL FREQUENCY	:	20MHZ
;**************************************************************************************
;AUTHOR			:	RAKESH PAREKH, MICROCHIP TECHNOLOGY INC.
;DATE			:	15-DEC-2003
;VERSION		:	V1.0
;APPLICATION NOTE	:	AN889
;**************************************************************************************
;*
;* Software License Agreement
;*
;* The software supplied herewith by Microchip Technology Incorporated
;* (the Company) for its PICmicro Microcontroller is intended and
;* supplied to you, the Companys customer, for use solely and
;* exclusively on Microchip PICmicro Microcontroller products. The
;* software is owned by the Company and/or its supplier, and is
;* protected under applicable copyright laws. All rights are reserved.
;* Any use in violation of the foregoing restrictions may subject the
;* user to criminal sanctions under applicable laws, as well as to
;* civil liability for the breach of the terms and conditions of this
;* license.
;*
;* THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
;* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
;* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
;* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
;* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
;* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
;**************************************************************************************
;DESCRIPTION
;**************************************************************************************
;THIS FILE HAVE ROUTINES FOR SETTING OF ADC, CALCULATION OF PARAMETERS FOR
;ACCELERATION AND DECELERATION OF MOTOR AND 16BIT/8BIT UNSIGNED DIVISIOM
;THIS FILE SHOULD BE USED ALONG WITH VF_CONTROL_WITH_7X7.ASM IN A PROJECT
;**************************************************************************************
	LIST		P = P16F777
	INCLUDE		<P16F777.INC>
	INCLUDE		<PARAMETERS.INC>
;**************************************************************************************
	UDATA
NEW_FREQ		RES	1	;NEW REFERENCE FREQUENCY INPUT
FLAG_FREQ		RES	1	;BYTE USED FOR HOLDING FLAGS RELATED CHANGE IN FREQ. OPERATION
TEMP_FREQ		RES	1	;ACTS AS A BUFFER BETWEEN NEW REQUIRED FREQ AND SET_FREQ (REPLICA OF NEW_FREQ)
DEC_COUNTER_LSB		RES	1	;16BIT RESULT OF (DEC_CON/DIFF IN FREQ)
DEC_COUNTER_MSB		RES	1	;INDICATES NO. OF TMR2 OV INTERRUPTS AFTER WHICH SET_FREQ=SET_FREQ-1
ACC_COUNTER_LSB		RES	1	;16BIT RESULT OF (ACC_CON/DIFF IN FREQ)
ACC_COUNTER_MSB		RES	1	;INDICATES NO. OF TMR2 OV INTERRUPTS AFTER WHICH SET_FREQ=SET_FREQ+1
TEMP_COUNTER_LSB	RES	1	;TEMPORAY LOCATION FOR HOLDING DEC_COUNTER OR ACC_COUNTER
TEMP_COUNTER_MSB	RES	1	;TEMPORAY LOCATION FOR HOLDING DEC_COUNTER OR ACC_COUNTER

	GLOBAL	NEW_FREQ
	GLOBAL	FLAG_FREQ
	GLOBAL	DEC_COUNTER_LSB
	GLOBAL	DEC_COUNTER_MSB
	GLOBAL	ACC_COUNTER_LSB
	GLOBAL	ACC_COUNTER_MSB
	GLOBAL	TEMP_COUNTER_LSB
	GLOBAL	TEMP_COUNTER_MSB	
	GLOBAL	SET_ADC_GO
	GLOBAL	TEMP_FREQ
	
	EXTERN	NO_1_LSB
	EXTERN	NO_1_MSB
	EXTERN	NO_2_LSB
	EXTERN	NO_2_MSB
	EXTERN	SET_FREQ
	EXTERN	FLAGS
	EXTERN	FREQ_REF_H
	EXTERN	FREQ_REF_L
	EXTERN	TEMP_LOC
	EXTERN	TEMP_LOC_1
	EXTERN	TEMP_LOC_2
	EXTERN	RESULT_MSB
	EXTERN	RESULT_LSB
	
DIVMAC  MACRO				;MACRO FOR DIVISION
	LOCAL   NOCHK
	LOCAL   NOGO
	BCF     STATUS,C
	RLF     TEMP_LOC_2,F
	RLF     TEMP_LOC_1,F
	RLF     RESULT_LSB,F
	RLF     RESULT_MSB,F
	MOVF    NO_2_MSB,W
	SUBWF   RESULT_MSB,W
	BTFSS   STATUS,Z
	GOTO    NOCHK
	MOVF    NO_2_LSB,W
	SUBWF   RESULT_LSB,W
NOCHK
	BTFSS   STATUS,C
	GOTO    NOGO
	MOVF    NO_2_LSB,W
	SUBWF   RESULT_LSB,F
	BTFSS   STATUS,C
	DECF    RESULT_MSB,F
	MOVF    NO_2_MSB,W
	SUBWF   RESULT_MSB,F
	BSF     STATUS,C
NOGO
	RLF     NO_1_LSB,F
	RLF     NO_1_MSB,F
	ENDM				;END OF MACRO FOR DIVISION
;*******************************************************************************
;THIS ROUTINE SETS THE ADC GO BIT HIGH AFTER AN AQUISITION TIME OF 20US APPROX. 
;*******************************************************************************
PROG0	CODE
SET_ADC_GO
	BTFSS	FLAGS,MOTOR_RUNNING
	RETURN
	BANKSEL	ADCON0
	BTFSC	ADCON0,GO		;CHECK IS CONVERSION IN PROGRESS?
	RETURN				;YES
	
	BANKSEL	FLAG_FREQ
	BTFSS	FLAG_FREQ,DELAY_4MS	;CHECK IS 4MS DELAY OVER?
	GOTO	CONVERT_BUS_VOLTAGE	;NO - GO FOR SETTING ADC FOR READING DC BUS VOLTGAE
#IFDEF	RAMP
	GOTO	CONVERT_FREQUENCY	;YES - GO FOR SETTING ADC FOR READING FREQ. SETTING
#ELSE
	GOTO	EQUAL_FREQ
#ENDIF

#IFDEF	RAMP
CONVERT_FREQUENCY
	BCF	FLAG_FREQ,DELAY_4MS	;CLEAR 4MS DELAY FLAG
	
	BANKSEL	NEW_FREQ
	MOVF 	SET_FREQ,W
	SUBWF	NEW_FREQ,W		;CHECK ARE OLD FREQUENCY AND NEW FREQUENCY SAME?
	BTFSS	STATUS,Z					
	GOTO	CHECK_LAST_NEW_F	;NO

	BANKSEL	FLAG_FREQ
	BCF	FLAG_FREQ,CHANGE_FREQ
	GOTO	EQUAL_FREQ		;YES
	
CHECK_LAST_NEW_F
		
	BANKSEL	TEMP_FREQ
	MOVF	TEMP_FREQ,W
	BANKSEL	NEW_FREQ
	SUBWF	NEW_FREQ,W		;CHECK ARE LAST NEW FREQUENCY AND PRESENT NEW FREQUENCY SAME?
	BTFSS	STATUS,Z
	GOTO	ASSIGN_NEW_TO_TEMP	;NO
	
	BANKSEL	FLAG_FREQ		;YES
	BTFSC	FLAG_FREQ,CHANGE_FREQ	;CHECK IS CHANGE OF FREQUENCY IN PROGRESS?
	GOTO	EQUAL_FREQ				;YES - SKIP FREQUENCY RAMP CALCULATION?
	BSF	FLAG_FREQ,CHANGE_FREQ	;SET FLAG INDICATING CHANGE OF FREQ. HAS STARTED.
	
	BANKSEL	NEW_FREQ
	MOVF	SET_FREQ,W
	SUBWF	NEW_FREQ,W
	BTFSS	STATUS,C		;CHECK TO SEE WHAT IS NEEDED - ACCELERATION OR DECELERATION?
	GOTO	DECELERATE		;SET_FREQ > NEW_FREQ, GO FOR DECELERATION
	GOTO	ACCELERATE		;NEW_FREQ > SET_FREQ, GO FOR DECELERATION

DECELERATE
	BANKSEL	FLAG_FREQ
	BCF	FLAG_FREQ,DEC_ACC_F

	BANKSEL	NEW_FREQ
	MOVF	NEW_FREQ,W
	SUBWF	SET_FREQ,W
	MOVWF	NO_2_LSB		;STORE DIFFERENCE IN FREQUENCY AS DIVISIOR
	CLRF	NO_2_MSB
	
	MOVLW	LOW (DEC_CON)
	MOVWF	NO_1_LSB
	MOVLW	HIGH (DEC_CON)
	MOVWF	NO_1_MSB
	
	CALL	DIV_16X8		;CALCULATE STEP SIZE FOR DECELERATION=[DECELERATION CONSTANT/(SET_FREQ-NEW_FREQ)]

	BANKSEL	DEC_COUNTER_LSB		;STORE 16 BIT QUOTIENT IN DEC_COUNTER
	MOVF	NO_1_LSB,W		;AND ALSO IN TEMP_COUNTER
	MOVWF	DEC_COUNTER_LSB
	BANKSEL	DEC_COUNTER_MSB
	MOVF	NO_1_MSB,W
	MOVWF	DEC_COUNTER_MSB
	BANKSEL	TEMP_COUNTER_LSB
	MOVF	NO_1_LSB,W
	MOVWF	TEMP_COUNTER_LSB
	BANKSEL	TEMP_COUNTER_MSB
	MOVF	NO_1_MSB,W
	MOVWF	TEMP_COUNTER_MSB
	GOTO	EQUAL_FREQ
	
ACCELERATE
	BANKSEL	FLAG_FREQ
	BSF	FLAG_FREQ,DEC_ACC_F

	MOVWF	NO_2_LSB		;STORE DIFFERENCE IN FREQUENCY AS DISIOR
	CLRF	NO_2_MSB
	
	MOVLW	LOW (ACC_CON)
	MOVWF	NO_1_LSB
	MOVLW	HIGH (ACC_CON)
	MOVWF	NO_1_MSB
	
	CALL	DIV_16X8		;CALCULATE STEP SIZE FOR ACCELERATION=[ACCELERATION CONSTANT/(NEW_FREQ-SET_FREQ)]
	
	BANKSEL	ACC_COUNTER_LSB		;STORE 16 BIT COUNTER IN ACC_COUNTER
	MOVF	NO_1_LSB,W		;AND ALSO IN TEMP_COUNTER
	MOVWF	ACC_COUNTER_LSB
	BANKSEL	ACC_COUNTER_MSB
	MOVF	NO_1_MSB,W
	MOVWF	ACC_COUNTER_MSB
	BANKSEL	TEMP_COUNTER_LSB
	MOVF	NO_1_LSB,W
	MOVWF	TEMP_COUNTER_LSB
	BANKSEL	TEMP_COUNTER_MSB
	MOVF	NO_1_MSB,W
	MOVWF	TEMP_COUNTER_MSB
	GOTO	EQUAL_FREQ		
	
ASSIGN_NEW_TO_TEMP
	MOVF	NEW_FREQ,W		;ASSIGN VALUE OF NEW_FREQ TO TEMP_FREQ
	BANKSEL	TEMP_FREQ
	MOVWF	TEMP_FREQ
	BANKSEL	FLAG_FREQ
	BCF	FLAG_FREQ,CHANGE_FREQ	
#ENDIF

EQUAL_FREQ
#IFNDEF	RAMP
	MOVF	NEW_FREQ,W
	MOVWF	SET_FREQ
	BCF	FLAG_FREQ,DELAY_4MS
	BCF	FLAGS,V_F_FLAG		;CLEAR THIS FLAG TO INDICATE ADC WILL NOW DO CONVERSION FOR FREQUENCY
#ENDIF
	BCF	FLAGS,V_F_FLAG		;CLEAR THIS FLAG TO INDICATE ADC WILL NOW DO CONVERSION FOR FREQUENCY
	CALL	CALCULATE_FREQUENCY	;CALCULATE SAMPLE TIME FOR SINE_TABLE VALUE
	BANKSEL	ADCON0
	BCF	ADCON0,CHS1
	BSF	ADCON0,CHS0
	BSF	ADCON0,GO		;SET GO BIT FOR ADC CONVERSION START
	RETURN	

CONVERT_BUS_VOLTAGE
	BSF	FLAGS,V_F_FLAG		;SET THIS FLAG TO INDICATE ADC WILL NOW CONVERT DC BUS VOLTAGE
	BANKSEL	ADCON0
	MOVLW	0X91			;CONTROL WORD FOR STARTING AD CONVERSION OF RA2 - DC BUS VOLTAGE
	MOVWF	ADCON0
	BCF	ADCON0,CHS0
	BSF	ADCON0,CHS1
	BSF	ADCON0,GO		;SET GO BIT FOR ADC CONVERSION START
	RETURN		

;*******************************************************************************
;THIS ROUTINE CALCULATES THE TIMER1 RELOAD VALUE BASED ON RA1 READ VALUE (FREQ.)AND THE 
;SCALING FACTOR CALCULATED BASED ON THE MAIN CLOCK AND NUMBER OF SINE TABLE ENTRIES.
;TIMER0 VALUE = FFFF - (FREQUENCY_SCALE/FREQUENCY)	FREQ = (RA1 ADC RESULT)
;*******************************************************************************
CALCULATE_FREQUENCY
	MOVLW	HIGH(FREQ_SCALE)	;FREQUENCY_SCALE/FREQUENCY
	MOVWF	NO_1_MSB		;16 BIT BY 8 BIT DIVISION	     	
	MOVLW	LOW(FREQ_SCALE)	
	MOVWF	NO_1_LSB
	MOVF	SET_FREQ,W
	MOVWF	NO_2_LSB		;READ FREQUENCY FOR DENOMINATOR
	CLRF	NO_2_MSB
										
	CALL	DIV_16X8
					;RESULT IN NO_1_LSB AND NO_1_MSB
KEEP_RESULT_IN_RPM			;TIMER1 VALUE = FFFF-(FREQUENCY_SCALE/FREQUENCY) 
	BCF	STATUS,C
	RLF	NO_1_LSB,F		;MULTIPLY QUOTIENT by 2 TO GET EFFECTIVE
	RLF	NO_1_MSB,F		;TMR1 PRESCALER 1:16
	MOVF	NO_1_LSB,W
	SUBLW	0XFF
	MOVWF	FREQ_REF_L		;THE TIMER1 RELOAD VALUE STORED IN 
	MOVF	NO_1_MSB,W		;FREQ_REF_H & FREQ_REF_L 
	SUBLW	0XFF			;THESE VALUES WILL BE LOADED TO
	MOVWF	FREQ_REF_H		;TIMER1 IN TIMER1 OVERFLOW INTERRUPT
	RETURN
;*******************************************************************************
;ROUTINE FOR 16/8 BIT DIVISION
;*******************************************************************************
DIV_16X8
	CALL	SETUP
	CLRF    RESULT_MSB
	CLRF    RESULT_LSB

; USE THE DIVMAC MACRO 16 TIMES
	DIVMAC
	DIVMAC
	DIVMAC
	DIVMAC
	DIVMAC
	DIVMAC
	DIVMAC
	DIVMAC
	DIVMAC
	DIVMAC
	DIVMAC
	DIVMAC
	DIVMAC
	DIVMAC
	DIVMAC
	DIVMAC
	RETLW   0

SETUP   
	MOVLW   0X10             
	MOVWF   TEMP_LOC
	MOVF    NO_1_MSB,W         
	MOVWF   TEMP_LOC_1
	MOVF    NO_1_LSB,W
	MOVWF   TEMP_LOC_2
	CLRF    NO_1_MSB
	CLRF    NO_1_LSB
	RETLW   0		
;*******************************************************************************
;END OF FILE
;*******************************************************************************
	END
